/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <konkret/konkret.h>
#include "Cura_BindsToLANEndpoint.h"

#include "network.h"
#include "globals.h"
#include "port.h"
#include "endpoint.h"
#include "ref_factory.h"

static const CMPIBroker* _cb;

static void Cura_BindsToLANEndpointInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_BindsToLANEndpointCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_BindsToLANEndpointEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_BindsToLANEndpointEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    const Ports *ports = network_get_ports(network);
    Port *port;
    const Endpoints *endpoints;
    unsigned int j;
    for (unsigned int i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);

        CMPIObjectPath *lanEndpointOP = portToCIM_LANEndpointRefOP(port, _cb, ns);

        endpoints = network_get_endpoints_by_port(network, port);
        for (j = 0; j < endpoints_length(endpoints); ++j) {
            CMPIObjectPath *serviceAccessPointOP = endpointToCIM_ServiceAccessPointRefOP(endpoints_index(endpoints, j), _cb, ns);

            Cura_BindsToLANEndpoint w;
            Cura_BindsToLANEndpoint_Init(&w, _cb, ns);
            Cura_BindsToLANEndpoint_SetObjectPath_Dependent(&w, serviceAccessPointOP);
            Cura_BindsToLANEndpoint_SetObjectPath_Antecedent(&w, lanEndpointOP);
            Cura_BindsToLANEndpoint_Set_FrameType(&w, Cura_BindsToLANEndpoint_FrameType_Ethernet);

            KReturnInstance(cr, w);
        }
    }
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_BindsToLANEndpointGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_BindsToLANEndpointCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_BindsToLANEndpointModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_BindsToLANEndpointDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_BindsToLANEndpointExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static void Cura_BindsToLANEndpointAssociationInitialize(CMPIAssociationMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_BindsToLANEndpointAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_BindsToLANEndpointAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_BindsToLANEndpoint_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Cura_BindsToLANEndpointAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_BindsToLANEndpoint_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Cura_BindsToLANEndpointReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_BindsToLANEndpoint_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Cura_BindsToLANEndpointReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_BindsToLANEndpoint_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    Cura_BindsToLANEndpoint,
    Cura_BindsToLANEndpoint,
    _cb,
    Cura_BindsToLANEndpointInitialize(&mi))

CMAssociationMIStub(
    Cura_BindsToLANEndpoint,
    Cura_BindsToLANEndpoint,
    _cb,
    Cura_BindsToLANEndpointAssociationInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_BindsToLANEndpoint",
    "Cura_BindsToLANEndpoint",
    "instance association")
