/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "Cura_ElementSettingData.h"
#include <Cura_IPAssignmentSettingData.h>
#include "network.h"
#include "port.h"
#include "connection.h"
#include "endpoint.h"
#include "activeconnection.h"
#include "ref_factory.h"

static const CMPIBroker* _cb;

static void Cura_ElementSettingDataInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_ElementSettingDataCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_ElementSettingDataEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_ElementSettingDataEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    const Connections *connections = network_get_connections(network);
    Connection *connection;
    const Endpoints *endpoints;
    Endpoint *endpoint;
    const Ports *allPorts = network_get_ports(network);
    Ports *ports;
    unsigned int j, k;

    ActiveConnections *activeConnections = network_get_active_connections(network);
    bool portActive = false;

    for (unsigned int i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);

        CMPIObjectPath *settingDataOP_generic = connectionToCIM_IPAssignmentSettingDataRefOP(connection, _cb, ns);

        // Little hack -- create list of ports for only port we have so it can be processed the same
        if (connection_get_port(connection)) {
            ports = ports_new(1);
            ports_add(ports, connection_get_port(connection));
        } else {
            ports = (Ports *) allPorts;
        }

        for (j = 0; j < ports_length(ports); ++j) {
            portActive = active_connections_is_connection_active_on_port(activeConnections, connection, ports_index(ports, j));

            endpoints = network_get_endpoints_by_port(network, ports_index(ports, j));
            for (k = 0; k < endpoints_length(endpoints); ++k) {
                endpoint = endpoints_index(endpoints, k);

                CMPIObjectPath *managedElementOP = endpointToCura_IPProtocolEndpointRefOP(endpoint, _cb, ns);

                Cura_ElementSettingData elementSettingData;
                Cura_ElementSettingData_Init(&elementSettingData, _cb, ns);
                Cura_ElementSettingData_SetObjectPath_ManagedElement(&elementSettingData, managedElementOP);
                Cura_ElementSettingData_SetObjectPath_SettingData(&elementSettingData, settingDataOP_generic);
                Cura_ElementSettingData_Set_IsCurrent(&elementSettingData,
                                                        portActive ?
                                                        Cura_ElementSettingData_IsCurrent_Is_Current :
                                                        Cura_ElementSettingData_IsCurrent_Is_Not_Current);
                Cura_ElementSettingData_Set_IsDefault(&elementSettingData,
                                                        connection_get_autoconnect(connection) ?
                                                        Cura_ElementSettingData_IsDefault_Is_Default :
                                                        Cura_ElementSettingData_IsDefault_Is_Not_Default);
                KReturnInstance(cr, elementSettingData);
            }
        }
        if (connection_get_port(connection)) {
            // Free only if it has been allocated here
            free(ports);
        }
    }
    active_connections_free(activeConnections);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_ElementSettingDataGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_ElementSettingDataCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_ElementSettingDataModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_ElementSettingDataDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_ElementSettingDataExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static void Cura_ElementSettingDataAssociationInitialize(CMPIAssociationMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_ElementSettingDataAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_ElementSettingDataAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_ElementSettingData_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Cura_ElementSettingDataAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_ElementSettingData_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Cura_ElementSettingDataReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_ElementSettingData_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Cura_ElementSettingDataReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_ElementSettingData_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    Cura_ElementSettingData,
    Cura_ElementSettingData,
    _cb,
    Cura_ElementSettingDataInitialize(&mi))

CMAssociationMIStub(
    Cura_ElementSettingData,
    Cura_ElementSettingData,
    _cb,
    Cura_ElementSettingDataAssociationInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_ElementSettingData",
    "Cura_ElementSettingData",
    "instance association")
