/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <konkret/konkret.h>
#include "Cura_EthernetPort.h"

#include "network.h"
#include "globals.h"
#include "port.h"

static const CMPIBroker* _cb = NULL;

static void Cura_EthernetPortInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_EthernetPortCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_EthernetPortEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_EthernetPortEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    const Ports *ports = network_get_ports(network);
    Port *port;
    for (unsigned int i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);

        Cura_EthernetPort w;
        Cura_EthernetPort_Init(&w, _cb, ns);

        Cura_EthernetPort_Set_SystemName(&w, get_system_name());
        Cura_EthernetPort_Set_CreationClassName(&w, "Cura_EthernetPort");
        Cura_EthernetPort_Set_SystemCreationClassName(&w, get_system_creationg_class_name());
        Cura_EthernetPort_Set_DeviceID(&w, port_get_id(port));

        switch (port_get_state(port)) {
            case STATE_NA:
                Cura_EthernetPort_Set_EnabledState(&w, Cura_EthernetPort_EnabledState_Not_Applicable);
                break;
            case STATE_SERVICING:
            case STATE_COMPLETED:
                Cura_EthernetPort_Set_EnabledState(&w, Cura_EthernetPort_EnabledState_Enabled);
                break;
            case STATE_STARTING:
                Cura_EthernetPort_Set_EnabledState(&w, Cura_EthernetPort_EnabledState_Starting);
                break;
            case STATE_STOPPING:
            case STATE_SHUTTING_DOWN:
                Cura_EthernetPort_Set_EnabledState(&w, Cura_EthernetPort_EnabledState_Shutting_Down);
                break;
            case STATE_STOPPED:
            case STATE_DORMANT:
                Cura_EthernetPort_Set_EnabledState(&w, Cura_EthernetPort_EnabledState_Disabled);
                break;
            case STATE_ABORTED:
                Cura_EthernetPort_Set_EnabledState(&w, Cura_EthernetPort_EnabledState_Disabled);
                break;
            default:
                Cura_EthernetPort_Set_EnabledState(&w, Cura_EthernetPort_EnabledState_Unknown);
        }
        Cura_EthernetPort_Init_AvailableRequestedStates(&w, 2);
        Cura_EthernetPort_Set_AvailableRequestedStates(&w, 0, Cura_EthernetPort_AvailableRequestedStates_Enabled);
        Cura_EthernetPort_Set_AvailableRequestedStates(&w, 1, Cura_EthernetPort_AvailableRequestedStates_Disabled);
        KReturnInstance(cr, w);
    }
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_EthernetPortGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_EthernetPortCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_EthernetPortModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_EthernetPortDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_EthernetPortExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    Cura_EthernetPort,
    Cura_EthernetPort,
    _cb,
    Cura_EthernetPortInitialize(&mi))

static void Cura_EthernetPortMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_EthernetPortMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_EthernetPortInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return Cura_EthernetPort_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    Cura_EthernetPort,
    Cura_EthernetPort,
    _cb,
    Cura_EthernetPortMethodInitialize(&mi))

KUint32 Cura_EthernetPort_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_EthernetPortRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    Network *network = mi->hdl;
    KUint32 result = KUINT32_INIT;
    GError *err = NULL;

    Port *port = NULL;
    const Ports *ports = network_get_ports(network);
    for (unsigned int i = 0; i < ports_length(ports); ++i) {
        if (strcmp(port_get_id(ports_index(ports, i)), self->DeviceID.chars) == 0) {
            port = ports_index(ports, i);
        }
    }

    Require(RequestedState, "No state has been requested");

    switch (RequestedState->value) {
        case Cura_EthernetPort_RequestedState_Disabled:
            network_port_set_state(network, port, STATE_STOPPED, &err);
            break;
        case Cura_EthernetPort_RequestedState_Enabled:
            network_port_set_state(network, port, STATE_SERVICING, &err);
            break;
        default:
            KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "Invalid state requested");
            KUint32_Set(&result, 1);
            return result;
    }

    if (err != NULL) {
        error(err->message);
        KSetStatus2(_cb, status, ERR_FAILED, err->message);
        KUint32_Set(&result, err->code);
        return result;
    }

    KSetStatus(status, OK);
    KUint32_Set(&result, 0);
    return result;
}

KUint32 Cura_EthernetPort_SetPowerState(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_EthernetPortRef* self,
    const KUint16* PowerState,
    const KDateTime* Time,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 Cura_EthernetPort_Reset(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_EthernetPortRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 Cura_EthernetPort_EnableDevice(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_EthernetPortRef* self,
    const KBoolean* Enabled,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 Cura_EthernetPort_OnlineDevice(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_EthernetPortRef* self,
    const KBoolean* Online,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 Cura_EthernetPort_QuiesceDevice(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_EthernetPortRef* self,
    const KBoolean* Quiesce,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 Cura_EthernetPort_SaveProperties(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_EthernetPortRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 Cura_EthernetPort_RestoreProperties(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_EthernetPortRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_EthernetPort",
    "Cura_EthernetPort",
    "instance method")
