/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "Cura_IPProtocolEndpoint.h"

#include "network.h"
#include "port.h"
#include "endpoint.h"
#include "setting.h"
#include "connection.h"

static const CMPIBroker* _cb = NULL;

static void Cura_IPProtocolEndpointInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_IPProtocolEndpointCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_IPProtocolEndpointEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_IPProtocolEndpointEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    const Ports *ports = network_get_ports(network);
    const Endpoints *endpoints;
    Endpoint *endpoint;
    unsigned int j;
    for (unsigned int i = 0; i < ports_length(ports); ++i) {
        endpoints = network_get_endpoints_by_port(network, ports_index(ports, i));
        for (j = 0; j < endpoints_length(endpoints); ++j) {
            Cura_IPProtocolEndpoint w;
            endpoint = endpoints_index(endpoints, j);
            Cura_IPProtocolEndpoint_Init(&w, _cb, ns);
            Cura_IPProtocolEndpoint_Set_SystemName(&w, get_system_name());
            Cura_IPProtocolEndpoint_Set_SystemCreationClassName(&w, get_system_creationg_class_name());
            Cura_IPProtocolEndpoint_Set_CreationClassName(&w, "Cura_IPProtocolEndpoint");
            Cura_IPProtocolEndpoint_Set_Name(&w, endpoint_get_name(endpoint));

            switch (endpoint_get_type(endpoint)) {
                case IPv4:
                    Cura_IPProtocolEndpoint_Set_IPv4Address(&w, endpoint_get_ipv4address(endpoint));
                    Cura_IPProtocolEndpoint_Set_ProtocolIFType(&w, Cura_IPProtocolEndpoint_ProtocolIFType_IPv4);
                    break;
                case IPv6:
                    Cura_IPProtocolEndpoint_Set_IPv6Address(&w, endpoint_get_ipv6address(endpoint));
                    Cura_IPProtocolEndpoint_Set_ProtocolIFType(&w, Cura_IPProtocolEndpoint_ProtocolIFType_IPv6);
                    break;
                case IPv4_IPv6:
                    Cura_IPProtocolEndpoint_Set_IPv4Address(&w, endpoint_get_ipv4address(endpoint));
                    Cura_IPProtocolEndpoint_Set_IPv6Address(&w, endpoint_get_ipv6address(endpoint));
                    Cura_IPProtocolEndpoint_Set_ProtocolIFType(&w, Cura_IPProtocolEndpoint_ProtocolIFType_IPv4_v6);
                    break;
                default:
                    Cura_IPProtocolEndpoint_Null_IPv4Address(&w);
                    Cura_IPProtocolEndpoint_Null_IPv6Address(&w);
                    Cura_IPProtocolEndpoint_Set_ProtocolIFType(&w, Cura_IPProtocolEndpoint_ProtocolIFType_Other);
            }

            KReturnInstance(cr, w);
        }
    }
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_IPProtocolEndpointGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_IPProtocolEndpointCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_IPProtocolEndpointModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_IPProtocolEndpointDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_IPProtocolEndpointExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    Cura_IPProtocolEndpoint,
    Cura_IPProtocolEndpoint,
    _cb,
    Cura_IPProtocolEndpointInitialize(&mi))

static void Cura_IPProtocolEndpointMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_IPProtocolEndpointMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_IPProtocolEndpointInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return Cura_IPProtocolEndpoint_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    Cura_IPProtocolEndpoint,
    Cura_IPProtocolEndpoint,
    _cb,
    Cura_IPProtocolEndpointMethodInitialize(&mi))

KUint32 Cura_IPProtocolEndpoint_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_IPProtocolEndpointRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 Cura_IPProtocolEndpoint_BroadcastReset(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_IPProtocolEndpointRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_IPProtocolEndpoint",
    "Cura_IPProtocolEndpoint",
    "instance method")
