/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "Cura_OrderedComponent.h"
#include "network.h"
#include "connection.h"
#include "ref_factory.h"

static const CMPIBroker* _cb;

static void Cura_OrderedComponentInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_OrderedComponentCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_OrderedComponentEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_OrderedComponentEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    unsigned int j;
    Setting *setting;
    Connection *connection;
    const Connections *connections = network_get_connections(network);
    for (unsigned int i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);

        CMPIObjectPath *groupComponentOP = connectionToCIM_IPAssignmentSettingDataRefOP(connection, _cb, ns);

        for (j = 0; j < settings_length(connection_get_settings(connection)); ++j) {
            setting = settings_index(connection_get_settings(connection), j);

            CMPIObjectPath *partComponentOP = settingToCura_IPAssignmentSettingDataSubclassOP(setting, _cb, ns);

            Cura_OrderedComponent w;
            Cura_OrderedComponent_Init(&w, _cb, ns);
            Cura_OrderedComponent_SetObjectPath_GroupComponent(&w, groupComponentOP);
            Cura_OrderedComponent_SetObjectPath_PartComponent(&w, partComponentOP);
            Cura_OrderedComponent_Set_AssignedSequence(&w, 1);

            KReturnInstance(cr, w);
        }
    }
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_OrderedComponentGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_OrderedComponentCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_OrderedComponentModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_OrderedComponentDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_OrderedComponentExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static void Cura_OrderedComponentAssociationInitialize(CMPIAssociationMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_OrderedComponentAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_OrderedComponentAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_OrderedComponent_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Cura_OrderedComponentAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_OrderedComponent_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Cura_OrderedComponentReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_OrderedComponent_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Cura_OrderedComponentReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_OrderedComponent_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    Cura_OrderedComponent,
    Cura_OrderedComponent,
    _cb,
    Cura_OrderedComponentInitialize(&mi))

CMAssociationMIStub(
    Cura_OrderedComponent,
    Cura_OrderedComponent,
    _cb,
    Cura_OrderedComponentAssociationInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_OrderedComponent",
    "Cura_OrderedComponent",
    "instance association")
