/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "Cura_RemoteServiceAccessPoint.h"
#include "network.h"
#include "activeconnection.h"
#include "connection.h"
#include "setting.h"
#include "endpoint.h"

static const CMPIBroker* _cb = NULL;

static void Cura_RemoteServiceAccessPointInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_RemoteServiceAccessPointCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_RemoteServiceAccessPointEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_RemoteServiceAccessPointEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    unsigned int j;
    const Ports *ports = network_get_ports(network);
    Port *port;
    Endpoint *endpoint;
    const char *name;
    char *n;

    for (unsigned int i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);
        for (j = 0; j < endpoints_length(port_get_endpoints(port)); ++j) {
            endpoint = endpoints_index(port_get_endpoints(port), j);

            if (endpoint_get_type(endpoint) == Unknown || (endpoint_get_ipv4gateway(endpoint) == NULL && endpoint_get_ipv6gateway(endpoint) == NULL)) {
                continue;
            }

            name = endpoint_get_name(endpoint);
            Cura_RemoteServiceAccessPoint w;
            Cura_RemoteServiceAccessPoint_Init(&w, _cb, ns);
            Cura_RemoteServiceAccessPoint_Set_CreationClassName(&w, "Cura_RemoteServiceAccessPoint");
            Cura_RemoteServiceAccessPoint_Set_Name(&w, name);
            Cura_RemoteServiceAccessPoint_Set_SystemCreationClassName(&w, get_system_creationg_class_name());
            Cura_RemoteServiceAccessPoint_Set_SystemName(&w, get_system_name());
            Cura_RemoteServiceAccessPoint_Set_AccessContext(&w, Cura_RemoteServiceAccessPoint_AccessContext_Default_Gateway);
            switch (endpoint_get_type(endpoint)) {
                case IPv4:
                    Cura_RemoteServiceAccessPoint_Set_AccessInfo(&w, endpoint_get_ipv4gateway(endpoint));
                    Cura_RemoteServiceAccessPoint_Set_InfoFormat(&w, Cura_RemoteServiceAccessPoint_InfoFormat_IPv4_Address);
                    KReturnInstance(cr, w);
                    break;
                case IPv6:
                    Cura_RemoteServiceAccessPoint_Set_AccessInfo(&w, endpoint_get_ipv6gateway(endpoint));
                    Cura_RemoteServiceAccessPoint_Set_InfoFormat(&w, Cura_RemoteServiceAccessPoint_InfoFormat_IPv6_Address);
                    KReturnInstance(cr, w);
                    break;
                case IPv4_IPv6:
                    // TODO: test it
                    asprintf(&n, "%s IPv4", name);
                    Cura_RemoteServiceAccessPoint_Set_Name(&w, n);
                    Cura_RemoteServiceAccessPoint_Set_AccessInfo(&w, endpoint_get_ipv4gateway(endpoint));
                    Cura_RemoteServiceAccessPoint_Set_InfoFormat(&w, Cura_RemoteServiceAccessPoint_InfoFormat_IPv4_Address);
                    KReturnInstance(cr, w);
                    free(n);

                    asprintf(&n, "%s IPv6", name);
                    Cura_RemoteServiceAccessPoint_Set_Name(&w, n);
                    Cura_RemoteServiceAccessPoint_Set_AccessInfo(&w, endpoint_get_ipv6gateway(endpoint));
                    Cura_RemoteServiceAccessPoint_Set_InfoFormat(&w, Cura_RemoteServiceAccessPoint_InfoFormat_IPv6_Address);
                    KReturnInstance(cr, w);
                    free(n);
                default:
                    break;
            }
        }
    }
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_RemoteServiceAccessPointGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_RemoteServiceAccessPointCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_RemoteServiceAccessPointModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_RemoteServiceAccessPointDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_RemoteServiceAccessPointExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    Cura_RemoteServiceAccessPoint,
    Cura_RemoteServiceAccessPoint,
    _cb,
    Cura_RemoteServiceAccessPointInitialize(&mi))

static void Cura_RemoteServiceAccessPointMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_RemoteServiceAccessPointMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_RemoteServiceAccessPointInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return Cura_RemoteServiceAccessPoint_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    Cura_RemoteServiceAccessPoint,
    Cura_RemoteServiceAccessPoint,
    _cb,
    Cura_RemoteServiceAccessPointMethodInitialize(&mi))

KUint32 Cura_RemoteServiceAccessPoint_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const Cura_RemoteServiceAccessPointRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_RemoteServiceAccessPoint",
    "Cura_RemoteServiceAccessPoint",
    "instance method")
