/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "connection.h"
#include "connection_private.h"
#include "port_private.h"
#ifndef TESTING
#include "connection_nm.h"
#include "setting_nm.h"
#include <nm-setting-connection.h>
#include <nm-setting-ip4-config.h>
#include <nm-setting-ip6-config.h>
#endif
#include <string.h>
#include "network.h"

Connection *connection_new_from_NMConnection(Network *network, NMConnection *nmconnection)
{
    NMSettingConnection *settingConnection;
    NMSettingIP4Config *ipv4config;
    NMIP4Address *ipv4address;
    NMSettingIP6Config *ipv6config;
    NMIP6Address *ipv6address;
    unsigned int i;
    const char *method;
    Setting *setting;

    const char *id = nm_connection_get_uuid(nmconnection);
    const char *name = nm_connection_get_id(nmconnection);
    Connection *connection = connection_new(id, name);
    connection->priv = nmconnection;

    // Get MAC address
    const GByteArray *mac = NULL;
    if (nm_connection_is_type(nmconnection, NM_SETTING_WIRED_SETTING_NAME)) {
        mac = nm_setting_wired_get_mac_address((NMSettingWired *) nm_connection_get_setting(nmconnection, nm_setting_wired_get_type()));
    } else {
        // TODO: check all other connection types too
        warn("Unknown connection type");
    }
    if (mac != NULL) {
        char *macStr = macFromGByteArray(mac);
        const Ports *ports = network_get_ports(network);
        for (unsigned int i = 0; i < ports_length(ports); ++i) {
            if (strcmp(ports_index(ports, i)->mac, macStr) == 0) {
                connection->port = ports_index(ports, i);
                break;
            }
        }
        free(macStr);
    }

    settingConnection = nm_connection_get_setting_connection(nmconnection);
    connection->autoconnect = nm_setting_connection_get_autoconnect(settingConnection);
    if (settingConnection) {
        ipv4config = nm_connection_get_setting_ip4_config(nmconnection);
        if (ipv4config) {
            method = nm_setting_ip4_config_get_method(ipv4config);
            if (strcmp(method, "auto") == 0) {
                // DHCP
                setting = setting_new_DHCP(connection, IPv4);
                connection_add_setting(connection, setting);

            } else if (strcmp(method, "manual") == 0) {
                // Static address
                for (i = 0; i < nm_setting_ip4_config_get_num_addresses(ipv4config); ++i) {
                    ipv4address = nm_setting_ip4_config_get_address(ipv4config, i);
                    setting = setting_new_from_static_ipv4(connection, ipv4address, i);
                    connection_add_setting(connection, setting);
                }
            } else if (strcmp(method, "link-local") == 0) {
                // Link-local address
                setting = setting_new_link_local(connection, IPv4);
                connection_add_setting(connection, setting);
            } else if (strcmp(method, "disabled") == 0) {
                debug("IPv4 connection is disabled for connection: %s", connection->name);
            } else {
                warn("Unknown IPv4 method: %s", method);
            }
        }

        ipv6config = nm_connection_get_setting_ip6_config(nmconnection);
        if (ipv6config) {
            method = nm_setting_ip6_config_get_method(ipv6config);
            if (strcmp(method, "auto") == 0) {
                // DHCP
                setting = setting_new_DHCP(connection, IPv6);
                connection_add_setting(connection, setting);

            } else if (strcmp(method, "manual") == 0) {
                // Static or link-local addresses
                for (i = 0; i < nm_setting_ip6_config_get_num_addresses(ipv6config); ++i) {
                    ipv6address = nm_setting_ip6_config_get_address(ipv6config, i);
                    setting = setting_new_from_static_ipv6(connection, ipv6address, i);
                    connection_add_setting(connection, setting);
                }
            } else if (strcmp(method, "link-local") == 0) {
                // Link-local address
                setting = setting_new_link_local(connection, IPv6);
                connection_add_setting(connection, setting);
            } else if (strcmp(method, "ignore") == 0) {
                debug("IPv6 connection is disabled for connection: %s", connection->name);
            } else {
                warn("Unknown IPv6 method: %s", method);
            }
        }
    } else {
        warn("Unable to get setting connection");
    }
    return connection;
}
