/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "network.h"
#include "network_private.h"
#include "port.h"
#include "connection.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef CMPI_VER_ALL
#define HAS_CMPI
#endif

Network *_network = NULL;

#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>

void print_backtrace(int signal)
{
    fprintf(stderr, "BackTrace\n");
    int ppid = getpid();
    int pid = fork();
    if (pid == 0) {
        // Child process
        char *strpid;
        asprintf(&strpid, "%d", ppid);
        execl("/usr/bin/gdb", "/usr/bin/gdb", "-p", strpid, NULL);
    } else {
        int status;
        waitpid(pid, &status, 0);
        system("/usr/bin/pkill -9 sfcbd");
    }
}

Port *network_port_by_mac(Network *network, const char *mac)
{
    Port *port;
    for (unsigned int i = 0; i < ports_length(network->ports); ++i) {
        port = ports_index(network->ports, i);
        if (strcmp(port_get_mac(port), mac) == 0) {
            return port;
        }
    }
    return NULL;
}

void *network_thread_start(void *data)
{
    Network *network = data;
    network->thread_loop = g_main_loop_new(NULL, FALSE);

    // Create the list of the devices
    network_priv_get_devices(network);

    g_main_loop_run(network->thread_loop);
    g_main_loop_unref(network->thread_loop);
    return NULL;
}

Network *network_new(const CMPIBroker *broker)
{
    signal(SIGSEGV, print_backtrace);
    fprintf(stderr, "network_new, pid: %d\n", getpid());
    Network *network;
    network = malloc(sizeof(Network));
    network->loaded = 0;
    network->ref_count = 0;
    network->priv = network_priv_new(network);
#ifdef HAS_CMPI
    network->broker = broker;
    network->mutex = broker->xft->newMutex(0);
    network->thread = broker->xft->newThread(network_thread_start, network, 1);
#else
    pthread_mutex_init(&network->mutex, NULL);
    pthread_create(&network->thread, NULL, network_thread_start, network);
#endif
    network->ports = NULL;
    network->connections = NULL;
    return network;
}

void network_free(Network *network)
{
#ifdef HAS_CMPI
    network->broker->xft->destroyMutex(network->mutex);
#else
    pthread_mutex_destroy(&network->mutex);
#endif
    if (network->thread_loop) {
        g_main_loop_quit(network->thread_loop);
    }
    if (network->priv) {
        network_priv_free(network->priv);
    }
    if (network->ports) {
        ports_free(network->ports);
    }
    if (network->connections) {
        connections_free(network->connections);
    }
    free(network);
    network = NULL;
}

Network *network_ref(const CMPIBroker *broker)
{
    if (_network == NULL) {
        _network = network_new(broker);
    }
    while (_network->loaded != (NETWORK_DEVICES_LOADED | NETWORK_SETTINGS_LOADED)) {
        // We need to wait for other thread to fetch network devices and settings
        // TODO: don't use polling
        usleep(10000);
    }
    MUTEX_LOCK(_network);
    _network->ref_count++;
    MUTEX_UNLOCK(_network);
    return _network;
}

void network_unref(Network *network)
{
    if (network == NULL || _network == NULL) {
        network = _network = NULL;
        return;
    }

    MUTEX_LOCK(network);
    network->ref_count--;
    MUTEX_UNLOCK(network);

    if (network->ref_count <= 0) {
        network_free(network);
        network = _network = NULL;
    }
}

const Ports *network_get_ports(Network *network)
{
    return network->ports;
}

int network_port_set_state(Network *network, Port *port, PortState state, GError **err)
{
    return network_priv_port_set_state(network, port, state, err);
}

const Endpoints *network_get_endpoints_by_port(Network *network, Port *port)
{
    return port_get_endpoints(port);

}

const Connections *network_get_connections(Network *network)
{
    return network->connections;
}

int network_activate_connection(Network *network, const Port *port, const Connection *connection)
{
    return network_priv_activate_connection(network, port, connection);
}

ActiveConnections *network_get_active_connections(Network *network)
{
    return network_priv_get_active_connections(network);
}

int network_create_connection(Network *network, Connection *connection)
{
    return network_priv_create_connection(network, connection);
}

int network_delete_connection(Network *network, Connection *connection)
{
    return network_priv_delete_connection(network, connection);
}
