/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "network_nm.h"

#include <string.h>
#include <stdio.h>

#include "errors.h"
#include "network_private.h"
#include "port_private.h"
#include "port_nm.h"
#include "endpoint_nm.h"
#include "endpoint_private.h"
#include "setting_nm.h"
#include "connection.h"
#include "connection_nm.h"
#include "setting.h"
#include "activeconnection.h"
#include "activeconnection_private.h"

#ifndef TESTING
#include <nm-client.h>
#include <nm-remote-settings.h>
#include <nm-device-ethernet.h>
#include <nm-utils.h>
#endif

typedef struct _NetworkPriv {
    NMClient *nm_client;
    NMRemoteSettings *remoteSettings;
} NetworkPriv;

void *network_priv_new(Network *network)
{
    g_type_init();
    NetworkPriv *priv = malloc(sizeof(NetworkPriv));
    priv->nm_client = nm_client_new();
    priv->remoteSettings = NULL;
    return priv;
}

void network_priv_free(void *data)
{
    NetworkPriv *priv = (NetworkPriv *) data;
    if (priv->nm_client) {
        g_object_unref(priv->nm_client);
    }
    if (priv->remoteSettings) {
        g_object_unref(priv->remoteSettings);
    }
    free(priv);
}

void network_priv_connection_activated_cb(NMClient *client, const char *object_path, GError *error, Network *network)
{
    debug("Network connection activated");
}

int network_priv_activate_connection(Network *network, const Port *port, const Connection *connection)
{
    NMConnection *nmconnection = connection_get_priv(connection);
    NMClient *nmclient = ((NetworkPriv *) network->priv)->nm_client;
    NMDevice *nmdevice = port->priv;
    nm_client_activate_connection(nmclient, nmconnection, nmdevice, NULL, (NMClientActivateFn) network_priv_connection_activated_cb, network);
    return 0;
}

void network_priv_device_deactivated_cb(NMDevice *device, GError *error, Network *network)
{
    debug("Device %s deactivated", nm_device_get_iface(device));
}

int network_priv_activate_default_connection(Network *network, Port *port, GError **error)
{
    Connection *connection;
    const Connections *connections = network_get_connections(network);
    for (unsigned int i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);
        if (!connection_get_autoconnect(connection)) {
            continue;
        }
        if (port_compare(connection_get_port(connection), port)) {
            return network_activate_connection(network, port, connection);
        }
    }
    cim_set_error(error, "Port %s has no default connection.", port_get_id(port));
    return 1;
}

int network_priv_port_set_state(Network *network, Port *port, PortState state, GError **err)
{
    switch (state) {
        case STATE_STOPPED:
            nm_device_disconnect(port->priv, (NMDeviceDeactivateFn) network_priv_device_deactivated_cb, network);
            break;
        case STATE_SERVICING:
            return network_priv_activate_default_connection(network, port, err);
            break;
        default:
            return 1;
    }
    return 0;
}

ActiveConnections *network_priv_get_active_connections(Network *network)
{
    NetworkPriv *priv = network->priv;
    const GPtrArray *nmactiveConnections = nm_client_get_active_connections(priv->nm_client);
    NMActiveConnection *nmactiveConnection;
    const Connections *connections = network_get_connections(network);
    Connection *connection;

    ActiveConnections *activeConnections = active_connections_new(nmactiveConnections->len);
    ActiveConnection *activeConnection;
    unsigned int j, k;
    const char *path;
    const GPtrArray *devices;
    NMDevice *device;
    const Ports *ports;
    Port *port;
    for (unsigned int i = 0; i < nmactiveConnections->len; ++i) {
        nmactiveConnection = g_ptr_array_index(nmactiveConnections, i);
        path = nm_active_connection_get_connection(nmactiveConnection);

        activeConnection = active_connection_new();
        active_connections_add(activeConnections, activeConnection);

        for (j = 0; j < connections_length(connections); ++j) {
            connection = connections_index(connections, j);
            if (strcmp(nm_connection_get_path(connection_get_priv(connection)), path) == 0) {
                activeConnection->connection = connection;
                break;
            }
        }
        devices = nm_active_connection_get_devices(nmactiveConnection);
        ports = network_get_ports(network);
        activeConnection->ports = ports_new(devices->len);
        for (j = 0; j < devices->len; ++j) {
            device = g_ptr_array_index(devices, j);
            for (k = 0; k < ports_length(ports); ++k) {
                port = ports_index(ports, k);
                if (strcmp(port->id, nm_device_get_iface(device)) == 0) {
                    ports_add(activeConnection->ports, port);
                    break;
                }
            }
        }
    }
    return activeConnections;
}

Endpoints *device_get_endpoints(Network *network, Port *port)
{
    Endpoints *endpoints = endpoints_new(5);
    const GSList *addresses = NULL;
    Endpoint *endpoint;
    int id = 0;

    NMDevice *device = nm_client_get_device_by_iface(((NetworkPriv *) network->priv)->nm_client, port->id);
    NMIP4Config *ip4config = nm_device_get_ip4_config(device);
    NMIP6Config *ip6config = nm_device_get_ip6_config(device);

    if (ip4config) {
        addresses = nm_ip4_config_get_addresses(ip4config);
        while (addresses) {
            endpoint = endpoint_new_from_NMIP4Address(port, addresses->data);
            asprintf(&endpoint->name, "%s_%d", port->id, id++);
            endpoints_add(endpoints, endpoint);
            addresses = g_slist_next(addresses);
        }
    }

    // In order to minimize number of endpoint we will try to add both
    // IPv4 and IPv6 address to each endpoint
    unsigned int index = 0;
    if (ip6config) {
        addresses = nm_ip6_config_get_addresses(ip6config);

        while (addresses) {
            if (index < endpoints_length(endpoints)) {
                endpoint = endpoints_index(endpoints, index);
                endpoint->type = IPv4_IPv6;
                endpoint->ipv6address = ip6ToString(nm_ip6_address_get_address(addresses->data));
                endpoint->ipv6prefix = (uint8_t) nm_ip6_address_get_prefix(addresses->data);
                endpoint->ipv6gateway = ip6ToString(nm_ip6_address_get_gateway(addresses->data));
            } else {
                endpoint = endpoint_new_from_NMIP6Address(port, addresses->data);
                asprintf(&endpoint->name, "%s_%d", port->id, id++);
                endpoints_add(endpoints, endpoint);
            }
            addresses = g_slist_next(addresses);
            index++;
        }
    }

    if (endpoints_length(endpoints) == 0) {
        // Add at least fake endpoint if we don't have any
        endpoint = endpoint_new(port);
        asprintf(&endpoint->name, "%s_%d", port->id, id++);
        endpoint->type = Unknown;
        endpoints_add(endpoints, endpoint);
    }
    return endpoints;
}

void network_device_state_changed_cb(NMDevice *device, NMDeviceState new_state, NMDeviceState old_state, NMDeviceStateReason reason, Network *network)
{
    fprintf(stderr, "Device: %s - state changed from %d to %d (%d)\n", nm_device_get_iface(device), old_state, new_state, reason);
    Port *port;
    for (unsigned int i = 0; i < ports_length(network->ports); ++i) {
        port = ports_index(network->ports, i);
        if (strcmp(port->id, nm_device_get_iface(device)) == 0) {
            MUTEX_LOCK(network);
            port->state = port_state_from_NMDeviceState(new_state);
            endpoints_free(port->endpoints);
            port->endpoints = device_get_endpoints(network, port);
            MUTEX_UNLOCK(network);
            break;
        }
    }
}

void network_device_added_cb(NMClient *nm_client, NMDevice *device, Network *network)
{
    fprintf(stderr, "Device: %s - added\n", nm_device_get_iface(device));
    g_signal_connect(device, "state-changed", G_CALLBACK(network_device_state_changed_cb), network);
    Port *port = port_new_from_NMDevice(device);
    port->endpoints = device_get_endpoints(network, port);
    MUTEX_LOCK(network);
    ports_add(network->ports, port);
    MUTEX_UNLOCK(network);
}

void network_device_removed_cb(NMClient *nm_client, NMDevice *device, Network *network)
{
    fprintf(stderr, "Device: %s - removed\n", nm_device_get_iface(device));
    int i, len = ports_length(network->ports);
    for (i = 0; i < len; i++) {
        if (strcmp(ports_index(network->ports, i)->id, nm_device_get_iface(device)) == 0) {
            break;
        }
    }
    if (i < len) {
        MUTEX_LOCK(network);
        port_free(ports_pop(network->ports, i));
        MUTEX_UNLOCK(network);
    }
}

// void remote_connection_removed_cb(NMRemoteConnection *remoteConnection, Network *network);
// void remote_connection_updated_cb(NMRemoteConnection *remoteConnection, Network *network);

void network_priv_new_connection(Network *network, NMConnection *nmconnection)
{
    Connection *connection = connection_new_from_NMConnection(network, nmconnection);
    MUTEX_LOCK(network);
    connections_add(network->connections, connection);
    MUTEX_UNLOCK(network);
}

void remote_connection_removed_cb(NMRemoteConnection *remoteConnection, Network *network)
{
    NMConnection *nmconnection = (NMConnection *) remoteConnection;
    fprintf(stderr, "Setting %s - removed\n", nm_connection_get_uuid(nmconnection));
    Connection *connection;
    const char *uuid;
    for (unsigned int i = 0; i < connections_length(network->connections); ++i) {
        connection = connections_index(network->connections, i);
        uuid = nm_connection_get_uuid(nmconnection);
        if (strncmp(uuid, connection_get_id(connection), strlen(uuid)) == 0) {
            // Delete the setting
            MUTEX_LOCK(network);
            connection_free(connections_pop(network->connections, i));
            MUTEX_UNLOCK(network);
            i--;
        }
    }
}

void remote_connection_updated_cb(NMRemoteConnection *remoteConnection, Network *network)
{
    NMConnection *connection = (NMConnection *) remoteConnection;
    fprintf(stderr, "Setting %s - updated\n", nm_connection_get_uuid(connection));
    remote_connection_removed_cb(remoteConnection, network);
    network_priv_new_connection(network, connection);
}

void network_remote_settings_read_cb(NMRemoteSettings *remoteSettings, Network *network)
{
    // All settings has been read, mark it as loaded
    MUTEX_LOCK(network);
    network->loaded |= NETWORK_SETTINGS_LOADED;
    MUTEX_UNLOCK(network);
}

void network_remote_setting_new_connection_cb(NMRemoteSettings *remoteSettings, NMConnection *connection, Network *network)
{
    printf("New connection, %s\n", nm_connection_get_id(connection));
    NMRemoteConnection *remoteConnection = nm_remote_settings_get_connection_by_path(((NetworkPriv *) network->priv)->remoteSettings, nm_connection_get_path(connection));
    if (remoteConnection) {
        g_signal_connect(remoteConnection, "updated", G_CALLBACK(remote_connection_updated_cb), network);
        g_signal_connect(remoteConnection, "removed", G_CALLBACK(remote_connection_removed_cb), network);
    }

    network_priv_new_connection(network, connection);
}

void network_priv_get_devices(Network *network)
{
    NetworkPriv *priv = network->priv;
    NMDevice *device;
    const GPtrArray *devices = nm_client_get_devices(priv->nm_client);
    MUTEX_LOCK(network);
    network->ports = ports_new(devices->len);
    MUTEX_UNLOCK(network);
    for (unsigned int i = 0; i < devices->len; ++i) {
        device = g_ptr_array_index(devices, i);
        network_device_added_cb(priv->nm_client, device, network);
    }
    MUTEX_LOCK(network);
    network->loaded |= NETWORK_DEVICES_LOADED;
    MUTEX_UNLOCK(network);
    g_signal_connect(priv->nm_client, "device-added", G_CALLBACK(network_device_added_cb), network);
    g_signal_connect(priv->nm_client, "device-removed", G_CALLBACK(network_device_removed_cb), network);

    network->connections = connections_new(5);
    priv->remoteSettings = nm_remote_settings_new(NULL);
    g_signal_connect(priv->remoteSettings, NM_REMOTE_SETTINGS_CONNECTIONS_READ, G_CALLBACK(network_remote_settings_read_cb), network);
    g_signal_connect(priv->remoteSettings, NM_REMOTE_SETTINGS_NEW_CONNECTION, G_CALLBACK(network_remote_setting_new_connection_cb), network);
}

void network_priv_connection_created(NMRemoteSettings *remoteSettings, NMRemoteConnection *connection, GError *err, Network *network)
{
    if (err != NULL) {
        error("Creating connection failed: %s (%d)", err->message, err->code);
    } else {
        debug("Connection created.");
    }
}

int network_priv_create_connection(Network *network, Connection *connection)
{
    NetworkPriv *priv = network->priv;
    NMConnection *nmconnection = nm_connection_new();
    NMSettingWired *nmsettingwired = NM_SETTING_WIRED(nm_setting_wired_new());
    nm_connection_add_setting(nmconnection, NM_SETTING(nmsettingwired));

    NMSettingConnection *nmsettingconnection = NM_SETTING_CONNECTION(nm_setting_connection_new());
    char *uuid = nm_utils_uuid_generate();
    g_object_set(nmsettingconnection,
                 NM_SETTING_CONNECTION_ID, connection_get_name(connection),
                 NM_SETTING_CONNECTION_TYPE, NM_SETTING_WIRED_SETTING_NAME,
//                  NM_SETTING_CONNECTION_AUTOCONNECT, connection->autoconnect ? TRUE : FALSE,
                 NM_SETTING_CONNECTION_UUID, uuid,
                 NULL);
    g_free(uuid);

    if (connection_get_port(connection) != NULL) {
        GByteArray *array = macToGByteArray(connection_get_port(connection)->mac);
        g_object_set(nmsettingwired, NM_SETTING_WIRED_MAC_ADDRESS, array, NULL);
        g_byte_array_free(array, true);
    }

    nm_connection_add_setting(nmconnection, NM_SETTING(nmsettingconnection));

    bool hasIPv4 = false;
    bool hasIPv6 = false;
    Setting *setting;
    for (unsigned int i = 0; i < settings_length(connection_get_settings(connection)); ++i) {
        setting = settings_index(connection_get_settings(connection), i);

        const char *method = setting_get_nm_method(setting);
        if (method == NULL) {
            error("%s setting on connection %s has unknown type", setting_get_protocol_type(setting) ? "IPv4" : "IPv6", connection_get_name(connection));
        }

        if (setting_get_protocol_type(setting) == IPv4) {
            hasIPv4 = true;
            NMSettingIP4Config *ipv4config = (NMSettingIP4Config *) nm_setting_ip4_config_new();

            g_object_set(ipv4config, NM_SETTING_IP4_CONFIG_METHOD, method, NULL);

            if (setting_get_type(setting) != SETTING_DHCP) {
                NMIP4Address *ipv4address = nm_ip4_address_new();
                nm_ip4_address_set_address(ipv4address, ip4FromString(setting_get_address(setting)));
                nm_ip4_address_set_prefix(ipv4address, setting_get_prefix(setting));
                nm_ip4_address_set_gateway(ipv4address, ip4FromString(setting_get_default_gateway(setting)));
                nm_setting_ip4_config_add_address(ipv4config, ipv4address);
            }
            nm_connection_add_setting(nmconnection, (NMSetting *) ipv4config);
        } else if (setting_get_protocol_type(setting) == IPv6) {
            hasIPv6 = true;
            NMSettingIP6Config *ipv6config = (NMSettingIP6Config *) nm_setting_ip6_config_new();

            g_object_set(ipv6config, NM_SETTING_IP6_CONFIG_METHOD, method, NULL);

            if (setting_get_type(setting) != SETTING_DHCP) {
                NMIP6Address *ipv6address = nm_ip6_address_new();
                struct in6_addr *ip6 = ip6FromString(setting_get_address(setting));
                nm_ip6_address_set_address(ipv6address, ip6);
                nm_ip6_address_set_prefix(ipv6address, setting_get_prefix(setting));
                nm_ip6_address_set_gateway(ipv6address, ip6FromString(setting_get_default_gateway(setting)));
                nm_setting_ip6_config_add_address(ipv6config, ipv6address);
                free(ip6);
            }
            nm_connection_add_setting(nmconnection, (NMSetting *) ipv6config);
        }
    }

    // If connection doesn't have IPv4 connection, it'll be disabled explicitly
    if (!hasIPv4) {
        debug("IPv4 disabled");
        NMSettingIP4Config *ipv4config = (NMSettingIP4Config *) nm_setting_ip4_config_new();
        g_object_set(ipv4config, NM_SETTING_IP4_CONFIG_METHOD, NM_SETTING_IP4_CONFIG_METHOD_DISABLED, NULL);
        nm_connection_add_setting(nmconnection, (NMSetting *) ipv4config);
    }
    // Same for IPv6
    if (!hasIPv6) {
        debug("IPv6 disabled");
        NMSettingIP6Config *ipv6config = (NMSettingIP6Config *) nm_setting_ip6_config_new();
        g_object_set(ipv6config, NM_SETTING_IP4_CONFIG_METHOD, NM_SETTING_IP6_CONFIG_METHOD_IGNORE, NULL);
        nm_connection_add_setting(nmconnection, (NMSetting *) ipv6config);
    }

    return nm_remote_settings_add_connection(priv->remoteSettings, nmconnection, (NMRemoteSettingsAddConnectionFunc) network_priv_connection_created, network) ? 0 : 1;
}

void network_priv_connection_deleted(NMRemoteConnection *nmremoteconnection, GError *err, Network *network)
{
    if (err != NULL) {
        error("Deleting connection failed: %s (%d)", err->message, err->code);
    } else {
        debug("Connection deleted.");
    }
}

int network_priv_delete_connection(Network *network, Connection *connection)
{
    NMConnection *nmconnection = connection_get_priv(connection);
    NMRemoteConnection *nmremoteconnection = nm_remote_settings_get_connection_by_path(((NetworkPriv *) network->priv)->remoteSettings, nm_connection_get_path(nmconnection));
    nm_remote_connection_delete(nmremoteconnection, (NMRemoteConnectionDeleteFunc) network_priv_connection_deleted, network);
    return 0;
}
