/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "globals.h"
#include "setting.h"
#include "setting_private.h"

#include "connection.h"
#include <string.h>

Setting *setting_new(ProtocolType protocol_type, SettingType setting_type)
{
    Setting *setting = malloc(sizeof(Setting));
    setting->id = NULL;
    setting->caption = NULL;
    setting->protocol_type = protocol_type;
    setting->type = setting_type;
    setting->address = NULL;
    setting->prefix = 0;
    setting->default_gateway = NULL;
    setting->clientID = NULL;

    setting->dns_servers = NULL;
    setting->search_domains = NULL;

    return setting;
}

Setting *setting_new_init(ProtocolType protocol_type, SettingType setting_type, const char *address, uint8_t prefix, const char *default_gateway)
{
    Setting *setting = setting_new(protocol_type, setting_type);
    setting->address = strdup(address);
    setting->prefix = prefix;
    setting->default_gateway = strdup(default_gateway);
    return setting;
}

SettingType setting_get_type(const Setting *setting)
{
    return setting->type;
}

ProtocolType setting_get_protocol_type(const Setting *setting)
{
    return setting->protocol_type;
}

const char *setting_get_id(const Setting *setting)
{
    return setting->id;
}

const char *setting_get_caption(const Setting *setting)
{
    return setting->caption;
}

const char *setting_get_address(const Setting *setting)
{
    return setting->address;
}

uint8_t setting_get_prefix(const Setting *setting)
{
    return setting->prefix;
}

const char *setting_get_default_gateway(const Setting *setting)
{
    return setting->default_gateway;
}

const char *setting_get_clientID(const Setting *setting)
{
    return setting->clientID;
}

void setting_set_clientID(Setting *setting, const char *clientID)
{
    setting->clientID = strdup(clientID);
}

void setting_add_dns_server(Setting *setting, const char *dns_server)
{
    if (setting->dns_servers == NULL) {
        setting->dns_servers = dns_servers_new(3);
    }
    char *dns = strdup(dns_server);
    dns_servers_add(setting->dns_servers, &dns);
}

void setting_add_search_domain(Setting *setting, const char *search_domain)
{
    if (setting->search_domains == NULL) {
        setting->search_domains = search_domains_new(3);
    }
    char *domain = strdup(search_domain);
    dns_servers_add(setting->dns_servers, &domain);
}

void setting_free(Setting *setting)
{
    if (setting->id != NULL)
        free(setting->id);
    if (setting->caption != NULL)
        free(setting->caption);
    if (setting->address != NULL)
        free(setting->address);
    if (setting->default_gateway != NULL)
        free(setting->default_gateway);
    if (setting->clientID != NULL)
        free(setting->clientID);
    if (setting->dns_servers != NULL)
        dns_servers_free(setting->dns_servers);
    if (setting->search_domains != NULL)
        search_domains_free(setting->search_domains);
    free(setting);
}

void dns_server_free(DNSServer *dns)
{
    free(dns);
}

void search_domain_free(SearchDomian *domain)
{
    free(domain);
}


LIST_IMPL(Setting, setting)
LIST_IMPL(DNSServer, dns_server)
LIST_IMPL(SearchDomian, search_domain)
